## Step 5: Adding WHERE Clause Handling to the Parser

In this step we add filtering capabilities by adding `WHERE` clause parsing to our DB.

### 5.1 Update the Parser to Handle WHERE Clauses
- In the `src` directory, update `queryParser.js` to parse the `WHERE` clause from the SQL query.
The parser should be able to extract the condition in a simple format, like `field = value`.

Here's an updated implementation:

```javascript
// src/queryParser.js

function parseQuery(query) {
    const selectRegex = /SELECT (.+?) FROM (.+?)(?: WHERE (.*))?$/i;
    const match = query.match(selectRegex);

    if (match) {
        const [, fields, table, whereClause] = match;
        return {
            fields: fields.split(',').map(field => field.trim()),
            table: table.trim(),
            whereClause: whereClause ? whereClause.trim() : null
        };
    } else {
        throw new Error('Invalid query format');
    }
}

module.exports = parseQuery;
```

### 5.2 Run the Updated Tests
- Run the tests using `npm test`. Ensure all tests pass, particularly the new test for WHERE clause parsing.
- If you see your older test fail, update that to include `"whereClause": null` in the expected output.

### 5.3 Update the Execute Function to apply `WHERE` clauses
Modify `executeSELECTQuery` in `src/index.js` to filter results based on the `WHERE` clause.

Example implementation:

```javascript
// src/index.js

const parseQuery = require('./queryParser');
const readCSV = require('./csvReader');

async function executeSELECTQuery(query) {
    const { fields, table, whereClause } = parseQuery(query);
    const data = await readCSV(`${table}.csv`);
    
    // Filtering based on WHERE clause
    const filteredData = whereClause
        ? data.filter(row => {
            const [field, value] = whereClause.split('=').map(s => s.trim());
            return row[field] === value;
        })
        : data;

    // Selecting the specified fields
    return filteredData.map(row => {
        const selectedRow = {};
        fields.forEach(field => {
            selectedRow[field] = row[field];
        });
        return selectedRow;
    });
}

module.exports = executeSELECTQuery;
```

> **💡Ask yourself:** Is the above implementation case-insensitive?

### 5.3 Update Tests for `WHERE` Clause
Modify the tests in `tests/index.test.js` to include tests for queries with `WHERE` clauses.

Example test:
```javascript
// tests/index.test.js

const executeSELECTQuery = require('../src/index');

test('Execute SQL Query with WHERE Clause', async () => {
    const query = 'SELECT id, name FROM sample WHERE age = 25';
    const result = await executeSELECTQuery(query);
    expect(result.length).toBe(1);
    expect(result[0]).toHaveProperty('id');
    expect(result[0]).toHaveProperty('name');
    expect(result[0].id).toBe('2');
});
```

> **💡Do it yourself:** Add tests where the above 
implementation would fail and to check for its case sensitivity/insensitivity.

### 5.4 Run the Updated Tests
- Run the tests using `npm test`


