## Step 7: Handling Different Comparison Operators

In this step we move further ahead from the realm on equalities to the realm of inequalities by implementing support for different comparison operators in our query parser.

### 7.1 Update the Parser to Recognize Comparison Operators
- Modify `parseWhereClause` function to handle a variety of comparison operators (`=`, `!=`, `>`, `<`, `>=`, `<=`). 
- The parser should identify the operator and split the condition accordingly.

Example implementation:

```javascript
// src/queryParser.js
function parseWhereClause(whereString) {
    const conditionRegex = /(.*?)(=|!=|>|<|>=|<=)(.*)/;
    return whereString.split(/ AND | OR /i).map(conditionString => {
        const match = conditionString.match(conditionRegex);
        if (match) {
            const [, field, operator, value] = match;
            return { field: field.trim(), operator, value: value.trim() };
        }
        throw new Error('Invalid WHERE clause format');
    });
}
```

### 7.2 Update the Execute Function to Apply Different Operators
- Modify `executeSELECTQuery` in `src/index.js` to evaluate conditions using the identified operators.

Example implementation:

```javascript
// src/index.js
function evaluateCondition(row, clause) {
    const { field, operator, value } = clause;
    switch (operator) {
        case '=': return row[field] === value;
        case '!=': return row[field] !== value;
        case '>': return row[field] > value;
        case '<': return row[field] < value;
        case '>=': return row[field] >= value;
        case '<=': return row[field] <= value;
        default: throw new Error(`Unsupported operator: ${operator}`);
    }
}
```
- Update the filteredData assignment to use the new `evaluateCondition` function.
```javascript
    const filteredData = whereClauses.length > 0
        ? data.filter(row => whereClauses.every(clause => evaluateCondition(row, clause)))
        : data;
```


### 7.3 Update Tests for Different Comparison Operators
Modify the tests in `tests/index.test.js` to include queries with different comparison operators and run the tests using `npm test`.

Example test:

```javascript
// tests/index.test.js
test('Execute SQL Query with Greater Than', async () => {
    const queryWithGT = 'SELECT id FROM sample WHERE age > 22';
    const result = await executeSELECTQuery(queryWithGT);
    expect(result.length).toEqual(2);
    expect(result[0]).toHaveProperty('id');
});

test('Execute SQL Query with Not Equal to', async () => {
    const queryWithGT = 'SELECT name FROM sample WHERE age != 25';
    const result = await executeSELECTQuery(queryWithGT);
    expect(result.length).toEqual(2);
    expect(result[0]).toHaveProperty('name');
});
```

> **💡Do it yourself:** Add negative test cases for the above implementation.