## Step 8: Inner Join

In this step we implement the `JOIN` functionality from SQL.

### 8.1 Preparing CSV Files for INNER JOIN
- Rename `sample.csv` to `student.csv`
- Update the tests in `tests/index.test.js` to use `student.csv` instead of `sample.csv` and update the expected results accordingly.

### 8.2 Create a new Enrollment CSV File
This setup gives us two CSV files: one for student information and another for their course enrollments. We can now use these files to demonstrate the INNER JOIN functionality in the next step.
Create a new file named `enrollment.csv` in the root of your project and add the following contents to it.

```csv
student_id,course
1,Mathematics
1,Physics
2,Chemistry
3,Mathematics
```

### 8.3 Update the Parser to Handle JOIN Clauses
- Define what the parser should output for a `JOIN` query.
Assuming the following `JOIN` query:
```sql
SELECT student.name, enrollment.course FROM student INNER JOIN enrollment ON student.id=enrollment.student_id
```
```json
{
    "fields": ["student.name", "enrollment.course"],
    "table": "student",
    "whereClauses": [],
    "joinTable": "enrollment",
    "joinCondition": {
        "left": "student.id",
        "right": "enrollment.student_id"
    }
}
```

- When there is a `WHERE` clause it would be like this:
```sql
SELECT student.name, enrollment.course FROM student INNER JOIN enrollment ON student.id=enrollment.student_id WHERE student.name = John
```

```json
{
    "fields": ["student.name", "enrollment.course"],
    "table": "student",
    "whereClauses": [{
        "field": "student.name",
        "operator": "=",
        "value": "John"
    }],
    "joinTable": "enrollment",
    "joinCondition": {
        "left": "student.id",
        "right": "enrollment.student_id"
    }
}
```

Now, modify `queryParser.js` to parse `JOIN` clauses in the SQL query.
The parser should identify the tables and the join condition.

Example implementation:

```javascript
// src/queryParser.js

function parseQuery(query) {
    // First, let's trim the query to remove any leading/trailing whitespaces
    query = query.trim();

    // Initialize variables for different parts of the query
    let selectPart, fromPart;

    // Split the query at the WHERE clause if it exists
    const whereSplit = query.split(/\sWHERE\s/i);
    query = whereSplit[0]; // Everything before WHERE clause

    // WHERE clause is the second part after splitting, if it exists
    const whereClause = whereSplit.length > 1 ? whereSplit[1].trim() : null;

    // Split the remaining query at the JOIN clause if it exists
    const joinSplit = query.split(/\sINNER JOIN\s/i);
    selectPart = joinSplit[0].trim(); // Everything before JOIN clause

    // JOIN clause is the second part after splitting, if it exists
    const joinPart = joinSplit.length > 1 ? joinSplit[1].trim() : null;

    // Parse the SELECT part
    const selectRegex = /^SELECT\s(.+?)\sFROM\s(.+)/i;
    const selectMatch = selectPart.match(selectRegex);
    if (!selectMatch) {
        throw new Error('Invalid SELECT format');
    }

    const [, fields, table] = selectMatch;

    // Parse the JOIN part if it exists
    let joinTable = null, joinCondition = null;
    if (joinPart) {
        const joinRegex = /^(.+?)\sON\s([\w.]+)\s*=\s*([\w.]+)/i;
        const joinMatch = joinPart.match(joinRegex);
        if (!joinMatch) {
            throw new Error('Invalid JOIN format');
        }

        joinTable = joinMatch[1].trim();
        joinCondition = {
            left: joinMatch[2].trim(),
            right: joinMatch[3].trim()
        };
    }

    // Parse the WHERE part if it exists
    let whereClauses = [];
    if (whereClause) {
        whereClauses = parseWhereClause(whereClause);
    }

    return {
        fields: fields.split(',').map(field => field.trim()),
        table: table.trim(),
        whereClauses,
        joinTable,
        joinCondition
    };
}
```

### 8.4 Update the Execute Function for INNER JOIN
Modify `executeSELECTQuery` in `src/index.js` to perform an `INNER JOIN` operation. Given that JOIN are generally performed before the where clause, we can apply the WHERE clause filtering after the JOIN operation.

Example implementation:
```javascript
// src/index.js at executeSELECTQuery

// Now we will have joinTable, joinCondition in the parsed query
const { fields, table, whereClauses, joinTable, joinCondition } = parseQuery(query);
let data = await readCSV(`${table}.csv`);

// Perform INNER JOIN if specified
if (joinTable && joinCondition) {
    const joinData = await readCSV(`${joinTable}.csv`);
    data = data.flatMap(mainRow => {
        return joinData
            .filter(joinRow => {
                const mainValue = mainRow[joinCondition.left.split('.')[1]];
                const joinValue = joinRow[joinCondition.right.split('.')[1]];
                return mainValue === joinValue;
            })
            .map(joinRow => {
                return fields.reduce((acc, field) => {
                    const [tableName, fieldName] = field.split('.');
                    acc[field] = tableName === table ? mainRow[fieldName] : joinRow[fieldName];
                    return acc;
                }, {});
            });
    });
}

// Apply WHERE clause filtering after JOIN (or on the original data if no join)
const filteredData = whereClauses.length > 0
    ? data.filter(row => whereClauses.every(clause => evaluateCondition(row, clause)))
    : data;
```

This would also require us to update the `SELECT` logic to handle the new `fields` format which should now work for both `SELECT` (`id`) and `SELECT JOIN` (`student.id`) queries. 

Sample implementation:
```javascript
// src/index.js at executeSELECTQuery

filteredData.map(row => {
    const selectedRow = {};
    fields.forEach(field => {
        // Assuming 'field' is just the column name without table prefix
        selectedRow[field] = row[field];
    });
    return selectedRow;
```

Verify if the results are as expected manually.

```json
/*
    result = [
      { 'student.name': 'John', 'enrollment.course': 'Mathematics' },
      { 'student.name': 'John', 'enrollment.course': 'Physics' },
      { 'student.name': 'Jane', 'enrollment.course': 'Chemistry' },
      { 'student.name': 'Bob', 'enrollment.course': 'Mathematics' }
    ]
*/
```

### 8.5 Update Tests for INNER JOIN

Modify the tests in `tests/index.test.js` to include tests for `INNER JOIN` queries. Add these 4 tests:

```javascript
test('Parse SQL Query with INNER JOIN', async () => {/*implement*/});
test('Parse SQL Query with INNER JOIN and WHERE Clause', async () => {/*implement*/});
test('Execute SQL Query with INNER JOIN', async () => {/*implement*/});
test('Execute SQL Query with INNER JOIN and a WHERE Clause', async () => {/*implement*/});
```

Make sure the tests pass. If you are unsure take a peek at the tests in the `08` commit.

> **💡Do it yourself:** Implement the tests mentioned in the code block above and also figure out negative test cases.
