## Step 9: Implementing LEFT and RIGHT JOINs

In this step we extend the `JOIN` functionality of our database to add support for `LEFT` and `RIGHT` joins as well.

### 9.0 Update the CSV Files
- Update the CSV Files to include the following data:

> `enrollment.csv` would become
```csv
student_id,course
1,Mathematics
1,Physics
2,Chemistry
3,Mathematics
5,Biology
```

and 
> `student.csv` would become
```csv
id,name,age
1,John,30
2,Jane,25
3,Bob,22
4,Alice,24
```

This will ensure that that the `JOIN` queries can now have data that is not present in both tables.

> 💡**Ask yourself**: Why was this update necessary?


### 9.1 Update the Parser to Recognize Different JOIN Types
- Modify `queryParser.js` to differentiate between `INNER JOIN`, `LEFT JOIN`, and `RIGHT JOIN`. Adjust the regex and logic to capture the type of JOIN.

Updated `queryParser.js`:

```javascript
// src/queryParser.js

// ...existing code...

function parseJoinClause(query) {
    const joinRegex = /\s(INNER|LEFT|RIGHT) JOIN\s(.+?)\sON\s([\w.]+)\s*=\s*([\w.]+)/i;
    const joinMatch = query.match(joinRegex);

    if (joinMatch) {
        return {
            joinType: joinMatch[1].trim(),
            joinTable: joinMatch[2].trim(),
            joinCondition: {
                left: joinMatch[3].trim(),
                right: joinMatch[4].trim()
            }
        };
    }

    return {
        joinType: null,
        joinTable: null,
        joinCondition: null
    };
}

// Update the parseQuery function to use parseJoinClause
// ...existing code...

module.exports = { parseQuery, parseJoinClause };
```

> 💡**Do it yourself**: Remember to update the `parseQuery` function to use the `parseJoinClause` function implemented above

### 9.2 Updating the Execute Function for Different JOIN Types
- Modify `executeSELECTQuery` in `src/index.js` to handle `LEFT JOIN` and `RIGHT JOIN` alongside `INNER JOIN`. Implement `performLeftJoin` and `performRightJoin` functions to encapsulate the specific logic for these JOIN types.

```javascript
// src/index.js

// ...existing imports...

// Helper functions for different JOIN types
function performInnerJoin(/* parameters */) {
    // Logic for INNER JOIN
    // ...
}

function performLeftJoin(/* parameters */) {
    // Logic for LEFT JOIN
    // ...
}

function performRightJoin(/* parameters */) {
    // Logic for RIGHT JOIN
    // ...
}

async function executeSELECTQuery(query) {
    const { fields, table, whereClauses, joinType, joinTable, joinCondition } = parseQuery(query);
    let data = await readCSV(`${table}.csv`);

    // Logic for applying JOINs
    if (joinTable && joinCondition) {
        const joinData = await readCSV(`${joinTable}.csv`);
        switch (joinType.toUpperCase()) {
            case 'INNER':
                data = performInnerJoin(data, joinData, joinCondition, fields, table);
                break;
            case 'LEFT':
                data = performLeftJoin(data, joinData, joinCondition, fields, table);
                break;
            case 'RIGHT':
                data = performRightJoin(data, joinData, joinCondition, fields, table);
                break;
            // Handle default case or unsupported JOIN types
        }
    }

    // ...existing code for WHERE clause and field selection...
}

module.exports = executeSELECTQuery;
```

> 💡**Do it yourself**: Implement the `performInnerJoin`, `performLeftJoin`, `performRightJoin` and update the `executeSELECTQuery` function. 

### 9.3 Adding Tests for LEFT and RIGHT JOINs
- Given the number of tests that we will have, would be good to refactor them in separate files. Consider creating `csvReader.test.js`, `queryExecutor.test.js`, and `queryParser.test.js` files in the `tests` folder and segrate the tests accordingly.

> 💡**Do it yourself**: Refactor tests

The following tests should pass now - [Link to the Commit](https://github.com/ChakshuGautam/stylusdb-sql/commit/7d4877d09055da7ef63ee6f2321db2e3fa87ad24)