## Step 11: Implementing ORDER BY Clause

In this step we implement the functionality of ordering our records in a particular sequence by adding support for the `ORDER BY` SQL clause.

### 11.1 Update the Parser to Handle ORDER BY Clauses
- Modify `queryParser.js` to parse `ORDER BY` clauses in the SQL query. The parser should identify the columns to order by and the sort direction (`ASC` or `DESC`).

Example snippet for updating `queryParser.js`:
```javascript
// src/queryParser.js

// ...existing code...

function parseQuery(query) {
    // ...existing parsing logic...

    // Updated regex to capture ORDER BY clause
    const orderByRegex = /\sORDER BY\s(.+)/i;
    const orderByMatch = query.match(orderByRegex);

    let orderByFields = null;
    if (orderByMatch) {
        orderByFields = orderByMatch[1].split(',').map(field => {
            const [fieldName, order] = field.trim().split(/\s+/);
            return { fieldName, order: order ? order.toUpperCase() : 'ASC' };
        });
    }

    return {
        // ...existing parsed parts,
        orderByFields
    };
}

// ...remaining code...

module.exports = { parseQuery, parseJoinClause };
```

> 💡 **Do it yourself**: Update the `parseQuery` function to add support for `ORDER BY` clause with the help of hints in the above code snippet.

> 💡 **Ask yourself**: Is there some scope of refactoring in the above code snippet?


### 11.2 Update the Execute Function to Apply ORDER BY Clause
- Modify `executeSELECTQuery` in `src/index.js` to sort the results based on the `ORDER BY` clause.

```javascript
// src/index.js

// ...existing imports and functions...

async function executeSELECTQuery(query) {
    const { fields, table, whereClauses, joinType, joinTable, joinCondition, groupByFields, orderByFields } = parseQuery(query);
    let data = await readCSV(`${table}.csv`);

    // ...existing logic for JOINs, WHERE clause, and GROUP BY...

    if (orderByFields) {
        data.sort((a, b) => {
            for (let { fieldName, order } of orderByFields) {
                if (a[fieldName] < b[fieldName]) return order === 'ASC' ? -1 : 1;
                if (a[fieldName] > b[fieldName]) return order === 'ASC' ? 1 : -1;
            }
            return 0;
        });
    }

    // ...existing logic for field selection...
}

module.exports = executeSELECTQuery;
```

> 💡 **Do it yourself**: Update the `executeSELECTQuery` function to add support for `ORDER BY` clause with the help of hints in the above code snippet.

### 11.3 Add Tests for ORDER BY Clause

- Add tests for the `ORDER BY` clause. You can see the existing tests [here](https://github.com/ChakshuGautam/stylusdb-sql/commit/39efbc7d7a81296c58a31e5fe84224938f64bcf7) to do TDD.

> 💡 **Ask yourself**: What is Test Drive Development? 