## Step 14: Implementing DISTINCT Keyword
### 14.1 Update the Parser to Handle DISTINCT
- Modify parseQuery in `queryParser.js` to recognize the `DISTINCT` keyword in the SQL query. The parser should identify when `DISTINCT` is used and modify the parsed query accordingly.

Example updates for queryParser.js:
```javascript
// src/queryParser.js

// Find were to add this code
if (query.toUpperCase().includes('SELECT DISTINCT')) {
    isDistinct = true;
    query = query.replace('SELECT DISTINCT', 'SELECT');
}
```

### 14.2 Update the Execute Function to Apply DISTINCT
Modify `executeSELECTQuery` in `src/index.js` to apply the DISTINCT keyword, removing duplicates from the result set.
Example updates for `executeSELECTQuery`:

```javascript
// src/index.js

// Find were to add this code
if (isDistinct) {
    data = [...new Map(data.map(item => [fields.map(field => item[field]).join('|'), item])).values()];
}

module.exports = executeSELECTQuery;
```

### 14.3 Add Tests for DISTINCT Keyword
Update the `enrollment.csv` to include `5,Physics,` and `student.csv` to include `5,Jane,22`. This will ensure the following tests pass now.

```javascript
test('Parse SQL Query with Basic DISTINCT', () => {
    const query = 'SELECT DISTINCT age FROM student';
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['age'],
        table: 'student',
        isDistinct: true,
        whereClauses: [],
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        orderByFields: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Parse SQL Query with DISTINCT and Multiple Columns', () => {
    const query = 'SELECT DISTINCT student_id, course FROM enrollment';
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['student_id', 'course'],
        table: 'enrollment',
        isDistinct: true,
        whereClauses: [],
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        orderByFields: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Parse SQL Query with DISTINCT and WHERE Clause', () => {
    const query = 'SELECT DISTINCT course FROM enrollment WHERE student_id = "1"';
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['course'],
        table: 'enrollment',
        isDistinct: true,
        whereClauses: [{ field: 'student_id', operator: '=', value: '"1"' }],
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        orderByFields: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Parse SQL Query with DISTINCT and JOIN Operations', () => {
    const query = 'SELECT DISTINCT student.name FROM student INNER JOIN enrollment ON student.id = enrollment.student_id';
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['student.name'],
        table: 'student',
        isDistinct: true,
        whereClauses: [],
        groupByFields: null,
        joinType: 'INNER',
        joinTable: 'enrollment',
        joinCondition: {
            left: 'student.id',
            right: 'enrollment.student_id'
        },
        orderByFields: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Parse SQL Query with DISTINCT, ORDER BY, and LIMIT', () => {
    const query = 'SELECT DISTINCT age FROM student ORDER BY age DESC LIMIT 2';
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['age'],
        table: 'student',
        isDistinct: true,
        whereClauses: [],
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        orderByFields: [{ fieldName: 'age', order: 'DESC' }],
        limit: 2,
        hasAggregateWithoutGroupBy: false
    });
});

test('Parse SQL Query with DISTINCT on All Columns', () => {
    const query = 'SELECT DISTINCT * FROM student';
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['*'],
        table: 'student',
        isDistinct: true,
        whereClauses: [],
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        orderByFields: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Basic DISTINCT Usage', async () => {
    const query = 'SELECT DISTINCT age FROM student';
    const result = await executeSELECTQuery(query);
    expect(result).toEqual([{ age: '30' }, { age: '25' }, { age: '22' }, { age: '24' }]);
});

test('DISTINCT with Multiple Columns', async () => {
    const query = 'SELECT DISTINCT student_id, course FROM enrollment';
    const result = await executeSELECTQuery(query);
    // Expecting unique combinations of student_id and course
    expect(result).toEqual([
        { student_id: '1', course: 'Mathematics' },
        { student_id: '1', course: 'Physics' },
        { student_id: '2', course: 'Chemistry' },
        { student_id: '3', course: 'Mathematics' },
        { student_id: '5', course: 'Biology' },
        { student_id: '5', course: 'Physics' }
    ]);
});

// Not a good test right now
test('DISTINCT with WHERE Clause', async () => {
    const query = 'SELECT DISTINCT course FROM enrollment WHERE student_id = "1"';
    const result = await executeSELECTQuery(query);
    // Expecting courses taken by student with ID 1
    expect(result).toEqual([{ course: 'Mathematics' }, { course: 'Physics' }]);
});

test('DISTINCT with JOIN Operations', async () => {
    const query = 'SELECT DISTINCT student.name FROM student INNER JOIN enrollment ON student.id = enrollment.student_id';
    const result = await executeSELECTQuery(query);
    // Expecting names of students who are enrolled in any course
    expect(result).toEqual([{ "student.name": 'John' }, { "student.name": 'Jane' }, { "student.name": 'Bob' }]);
});

test('DISTINCT with ORDER BY and LIMIT', async () => {
    const query = 'SELECT DISTINCT age FROM student ORDER BY age DESC LIMIT 2';
    const result = await executeSELECTQuery(query);
    // Expecting the two highest unique ages
    expect(result).toEqual([{ age: '30' }, { age: '25' }]);
});
```

Update all the others to ensure they pass as well.