## Step 15: Adding Support for LIKE Operator
### 15.1 Update the Parser to Handle LIKE Clauses
Modify `parseQuery` in `queryParser.js` to recognize `LIKE`` conditions in the `WHERE`` clause.

The parser should be capable of identifying LIKE patterns within the query.
Example updates for queryParser.js:

```javascript
if (conditionString.includes(' LIKE ')) {
    const [field, , pattern] = conditionString.split(/\sLIKE\s/i);
    return { field: field.trim(), operator: 'LIKE', value: pattern.trim() };
}
```

### 15.2 Update the Execute Function to Apply LIKE Conditions
Modify `executeSELECTQuery` in `src/index.js` to filter data using the LIKE operator.
Example updates for `executeSELECTQuery`:

```javascript
// Inside operator matching logic
if (clause.operator === 'LIKE') {
    // Transform SQL LIKE pattern to JavaScript RegExp pattern
    const regexPattern = '^' + clause.value.replace(/%/g, '.*') + '$';
    return new RegExp(regexPattern, 'i').test(row[clause.field]);
}
```

### 15.3 Add Tests for LIKE Operator
Make sure the following test passes now.

```javascript
test('Parse SQL Query with LIKE Clause', () => {
    const query = "SELECT name FROM student WHERE name LIKE '%Jane%'";
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['name'],
        table: 'student',
        whereClauses: [{ field: 'name', operator: 'LIKE', value: '%Jane%' }],
        isDistinct: false,
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        orderByFields: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Parse SQL Query with LIKE Clause and Wildcards', () => {
    const query = "SELECT name FROM student WHERE name LIKE 'J%'";
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['name'],
        table: 'student',
        whereClauses: [{ field: 'name', operator: 'LIKE', value: 'J%' }],
        isDistinct: false,
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        orderByFields: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Parse SQL Query with Multiple LIKE Clauses', () => {
    const query = "SELECT name FROM student WHERE name LIKE 'J%' AND age LIKE '2%'";
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['name'],
        table: 'student',
        whereClauses: [
            { field: 'name', operator: 'LIKE', value: 'J%' },
            { field: 'age', operator: 'LIKE', value: '2%' }
        ],
        isDistinct: false,
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        orderByFields: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Parse SQL Query with LIKE and ORDER BY Clauses', () => {
    const query = "SELECT name FROM student WHERE name LIKE '%e%' ORDER BY age DESC";
    const parsed = parseQuery(query);
    expect(parsed).toEqual({
        fields: ['name'],
        table: 'student',
        whereClauses: [{ field: 'name', operator: 'LIKE', value: '%e%' }],
        orderByFields: [{ fieldName: 'age', order: 'DESC' }],
        isDistinct: false,
        groupByFields: null,
        joinType: null,
        joinTable: null,
        joinCondition: null,
        limit: null,
        hasAggregateWithoutGroupBy: false
    });
});

test('Execute SQL Query with LIKE Operator for Name', async () => {
    const query = "SELECT name FROM student WHERE name LIKE '%Jane%'";
    const result = await executeSELECTQuery(query);
    // Expecting names containing 'Jane'
    expect(result).toEqual([{ name: 'Jane' }, { name: 'Jane' }]);
});

test('Execute SQL Query with LIKE Operator and Wildcards', async () => {
    const query = "SELECT name FROM student WHERE name LIKE 'J%'";
    const result = await executeSELECTQuery(query);
    // Expecting names starting with 'J'
    expect(result).toEqual([{ name: 'John' }, { name: 'Jane' }, { name: 'Jane' }]);
});

test('Execute SQL Query with LIKE Operator Case Insensitive', async () => {
    const query = "SELECT name FROM student WHERE name LIKE '%bob%'";
    const result = await executeSELECTQuery(query);
    // Expecting names 'Bob' (case insensitive)
    expect(result).toEqual([{ name: 'Bob' }]);
});

test('Execute SQL Query with LIKE Operator and DISTINCT', async () => {
    const query = "SELECT DISTINCT name FROM student WHERE name LIKE '%e%'";
    const result = await executeSELECTQuery(query);
    // Expecting unique names containing 'e'
    expect(result).toEqual([{ name: 'Jane' }, { name: 'Alice' }]);
});

test('LIKE with ORDER BY and LIMIT', async () => {
    const query = "SELECT name FROM student WHERE name LIKE '%a%' ORDER BY name ASC LIMIT 2";
    const result = await executeSELECTQuery(query);
    // Expecting the first two names alphabetically that contain 'a'
    expect(result).toEqual([{ name: 'Alice' }, { name: 'Jane' }]);
});
```
